/* giFTui
 * Copyright (C) 2003 the giFTui team
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "main.h"

#include <stdio.h>
#include <unistd.h>

#include <libgift/libgift.h>
#include <glib.h>

#include "util.h"
#include "signal.h"
#include "configure.h"

#ifdef HAVE_GETOPT_H
# include <getopt.h>
#endif

#define GIFTUI_CONFIG(arg) ((GiftuiConfig_t *) (arg))

typedef struct _GiftuiConfig_t
{
	gchar *file;
	Config *conf;
	gboolean modified;
} GiftuiConfig_t;

/**/

static GList *giftui_config;

static void
giftui_print_help (const gchar *name)
{
	printf (_("USAGE : %s [OPTION]...\n\n"), name);
#ifdef HAVE_GETOPT_LONG
	printf (_("      -s, --server domain connect to this server\n"));
	printf (_("      -p, --port   port   connect to this port\n"));
	printf (_("      -k, --kill          kill the daemon when exiting\n"));
	printf (_("      -l, --launch        launch the daemon at startup\n"));
	printf (_("      -h, --help          print this screen\n"));
	printf (_("      -v, --version       print version of %s\n"), PACKAGE);
#elif HAVE_GETOPT
	printf (_("      -s server    connect to this server\n"));
	printf (_("      -p port      connect to this port\n"));
	printf (_("      -k           kill the daemon when exiting\n"));
	printf (_("      -l           launch the daemon at startup\n"));
	printf (_("      -h           print this screen\n"));
	printf (_("      -v           print version of %s\n"), PACKAGE);
#endif
	printf (_("Report any bug on http://giftui.tuxfamily.org.\n"));
	
	return;
}

static void
giftui_print_version (void)
{
	printf ("%s\n", VERSION);
	
	return;
}

/**/

static GiftuiConfig_t *
giftui_config_search (const gchar *file)
{
       	GList *e;
	
	g_return_val_if_fail (file != NULL, NULL);
	
	e = g_list_first (giftui_config);
	
	while (e && strcmp (GIFTUI_CONFIG (e->data)->file, file))
		e = e->next;
	
	if (e)
		return e->data;
	
	return NULL;
}

void
giftui_config_set_str (const gchar *keypath, const gchar *val)
{
	gchar *file, *cval, *key, *s;
	GiftuiConfig_t *c;
	
	g_return_if_fail (keypath != NULL);
	
	file = g_strdup (keypath);
	if (val)
		cval = g_strdup (val);
	else
		cval = NULL;
	
	if (!(s = strchr (file, ':')))
	{
		g_free (file);
		
		return;
	}
	key = s + 1;
	
	*s = '\0';
	
	if ((c = giftui_config_search (file)))
		config_set_str (c->conf, key, cval);
	
	g_free (file);
	c->modified = TRUE;
	
	giftui_signal_emit (keypath);
	
	return;
}

void
giftui_config_set_int (const gchar *keypath, int val)
{
	gchar *file, *key, *s;
	GiftuiConfig_t *c;
	
	g_return_if_fail (keypath != NULL);
	
	file = g_strdup (keypath);
	if (!(s = strchr (file, ':')))
	{
		g_free (file);
		
		return;
	}
	key = s + 1;
	
	*s = '\0';
	
	if ((c = giftui_config_search (file)))
		config_set_int (c->conf, key, val);
	
	g_free (file);
	c->modified = TRUE;
	
	giftui_signal_emit (keypath);
	
	return;
}

/**/

const gchar *
giftui_config_get_str (const gchar *keypath)
{
	gchar *file, *key, *s;
	GiftuiConfig_t *c;
	
	g_return_val_if_fail (keypath != NULL, NULL);
	
	file = g_strdup (keypath);
	if (!(s = strchr (file, ':')))
	{
		g_free (file);
		
		return NULL;
	}
	key = s + 1;
	
	*s = '\0';
	
	c = giftui_config_search (file);
	
	if (c)
		s = config_get_str (c->conf, key);
	else
		s = NULL;
	
	g_free (file);
	
	return s;
}

int
giftui_config_get_int (const gchar *keypath)
{
	gint v = 0;
	gchar *file, *key, *s;
	GiftuiConfig_t *c;
	
	g_return_val_if_fail (keypath != NULL, v);
	
	file = g_strdup (keypath);
	if (!(s = strchr (file, ':')))
	{
		g_free (file);
		
		return v;
	}
	key = s + 1;
	
	*s = '\0';
	
	c = giftui_config_search (file);
	
	if (c)
		v = config_get_int (c->conf, key);
	
	g_free (file);
	
	return v;
}

/**/

static gboolean
giftui_config_copy_from_default (const gchar *file)
{
	gint nb;
	gchar *def_file;
	gint in;
	
	g_return_val_if_fail (file != NULL, FALSE);
	
	def_file = g_strdup_printf ("%s/giftui.conf", DATADIR);
	if (def_file && ((in = open (def_file, O_RDONLY)) != -1))
	{
		gint out;
		
		if ((out = open (file, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP)) != -1)
		{
			gchar buf[100];
			
			while ((nb = read (in, buf, 100)))
			{				
				if (nb < 100)
					write (out, buf, nb);
				else
					write (out, buf, 100);
					
			}
			close (out);
		}
		
		close (in);
		g_free (def_file);
		
		return TRUE;
	}
	
	GIFTUI_PRINT_ERR ((_("Cannot open default configuration from %s.\n"), def_file));
	g_free (def_file);
	
	return FALSE;
}

/**/

static gboolean
giftui_config_read (GiftuiConfig_t *conf, gchar *file)
{
	g_return_val_if_fail (conf != NULL, FALSE);
	
	if (conf->conf)
		config_free (conf->conf);
	
	if (!file)
	{
		if ((conf->conf = gift_config_new (conf->file)))
			return TRUE;
	}
	else
	{
		if ((conf->conf = config_new (file)))
			return TRUE;
		
		GIFTUI_PRINT_ERR ((_("Cannot read configuration from %s.\n"), file));
	}
	
	return FALSE;
}

static void
giftui_config_write (GiftuiConfig_t *conf)
{
	g_return_if_fail (conf != NULL);
	g_return_if_fail (conf->conf != NULL);
	
	if (conf->modified)
		config_write (conf->conf);
	conf->modified = FALSE;
	
	return;
}

static void
giftui_config_free (GiftuiConfig_t *conf)
{
	g_return_if_fail (conf != NULL);
	
	if (conf->conf)
		config_free (conf->conf);
	if (conf->file)
		g_free (conf->file);
	g_free (conf);
	
	return;
}

/**/

void
giftui_config_init (void)
{
	giftui_config = NULL;
	
	return;
}

void
giftui_config_end (void)
{
	GList *tmp;
	
	giftui_config = g_list_first (giftui_config);
	
	while (giftui_config)
	{
		tmp = giftui_config;
		giftui_config_free (GIFTUI_CONFIG (giftui_config->data));
		giftui_config = giftui_config->next;
		g_list_free_1 (tmp);
	}
	
	return;
}

gboolean
giftui_config_load_from_args (gint argc, gchar *argv[])
{
	gint version = FALSE, help = FALSE, c;
#ifdef HAVE_GETOPT_LONG
	struct option long_options[] =
		{
			{ "help", no_argument, NULL, 'h'},
			{ "version", no_argument, NULL, 'v'},
			{ "launch", no_argument, NULL, 'l'},
			{ "server", required_argument, NULL, 's'},
			{ "port", required_argument, NULL, 'p'},
			{ NULL, 0, NULL, 0}
		};
#endif
	
#ifdef HAVE_GETOPT_LONG
	while ((c = getopt_long (argc, argv, "hvlp:s:", long_options, NULL)) != -1) 
#elif HAVE_GETOPT
	while ((c = getopt (argc, argv, "hvlp:s:")) != -1)
#endif
	{
		switch (c)
		{
		case 'h':
			help = TRUE;
			break;
			
		case 'v':
			version = TRUE;
			break;
			
		case 'l':
			giftui_config_set_int ("giftui:dameon/launch", TRUE);
			break;
			
		case 'k':
			giftui_config_set_int ("giftui:dameon/kill", TRUE);
			break;
			
		case 'p':
			giftui_config_set_int ("ui:daemon/port", atoi (optarg));
			break;
			
		case 's':
			giftui_config_set_str ("ui:daemon/host", optarg);
			break;
			
		default:
			giftui_print_help (argv[0]);
			return TRUE;
		}
	}
	/* Help ? */
	if (help)
		giftui_print_help (argv[0]);
	
	/* Version ? */
	if (version)
		giftui_print_version ();
	
	return (help || version);
}

gboolean
giftui_config_load (void)
{
	gchar *f;
	GiftuiConfig_t *conf;
	
	if (giftui_config)
		giftui_config_end ();
	
	
	conf = g_new0 (GiftuiConfig_t, 1);
	if (conf)
	{
		conf->file = g_strdup ("ui");
		giftui_config_read (conf, NULL);
		
		giftui_config = g_list_append (giftui_config, conf);
	}
	
	conf = g_new0 (GiftuiConfig_t, 1);
	if (conf)
	{
		conf->file = g_strdup ("giftui");
		f = g_build_filename (g_get_home_dir (), ".giFT", "ui", "giftui.conf", NULL);
		if (f && !(giftui_config_read (conf, f)))
		{
			if (!giftui_config_copy_from_default (f))
			{
				giftui_config_free (conf);
				g_free (f);
				
				return FALSE;
			}
			giftui_config_read (conf, f);
			g_free (f);
		}
		
		giftui_config = g_list_append (giftui_config, conf);
	}
	
	return TRUE;
}


void
giftui_config_save (void)
{
	GList *l = g_list_first (giftui_config);
	
	while (l)
	{
		giftui_config_write (GIFTUI_CONFIG (l->data));
		l = l->next;
	}
	
	return;
}

/* DATA */

static GData *datalist;

void
giftui_data_set (const gchar *keypath, gpointer value)
{
	g_datalist_set_data (&datalist, keypath, value);
	giftui_signal_emit (keypath);
	
	return;
}

void
giftui_data_set_full (const gchar *keypath, gpointer value, GDestroyNotify destroy_func)
{
	g_datalist_remove_no_notify (&datalist, keypath);
	g_datalist_set_data_full (&datalist, keypath, value, destroy_func);
	giftui_signal_emit (keypath);
	
	return;
}

gpointer
giftui_data_get (const gchar *keypath)
{
	return g_datalist_get_data (&datalist, keypath);
}

void
giftui_data_init (void)
{
	datalist = NULL;
	g_datalist_init (&datalist);
	
	return;
}

void
giftui_data_end (void)
{
	g_datalist_clear (&datalist);
}
